from gitlab import Gitlab


def post_comment(gl_config, content):
    gl = Gitlab(url=gl_config.server_url, private_token=gl_config.private_token)
    project = gl.projects.get(id=gl_config.project_id, lazy=True)
    mr = project.mergerequests.get(id=gl_config.mr_id, lazy=True)

    expected_comment = f"<!-- job: {gl_config.job_name} -->"
    body = expected_comment + content
    for note in mr.notes.list(get_all=True, iterator=True):
        if note.body.startswith(expected_comment):
            note.body = body
            note.save()
            break
    else:
        mr.notes.create({"body": body})



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gitlab_mr_note']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0', 'python-gitlab>=3.12.0,<4.0.0']

entry_points = \
{'console_scripts': ['gitlab-mr-note = gitlab_mr_note.cli:main']}

setup_kwargs = {
    'name': 'gitlab-mr-note',
    'version': '1.0.0',
    'description': '',
    'long_description': '# Post or Update Comments on Gitlab MR\n\nThe idea behind this project is to post content on the Gitlab MR which\ndynamically updates as new commits are made. In order to do that, we add a\ncomment at the start of our content to identify the job the content is from.\n\n## Usage\n\n- General Access Token from `Settings > Access Tokens` with `api` permissions.\n- Go to `Settings > CI/CD > Variables > Add Variables`\n- Set the key to `GITLAB_PRIVATE_TOKEN` and value to key generated in first step.\n- Set `Mask` flag and uncheck `Protect` flag.\n- Create a test like follows\n\n```yaml\ntest:\n  stage: test\n  script:\n    - pip install gitlab-mr-note\n    - echo hello world | gitlab-mr-note\n  only:\n    - merge_requests\n```\n\n`gitlab-mr-note` infers the details from environment variables set in the CI.\nFor manual usage, pass the following args.\n\n```\nUsage: gitlab-mr-note [OPTIONS]\n\nOptions:\n  -s, --server-url TEXT     Server URL of gitlab instance\n  -m, --mr-id TEXT          ID of the MR to comment on\n  -p, --project-id TEXT     ID of the Project\n  -j, --job-name TEXT       Job Name\n  -t, --private-token TEXT  Private Token\n  --help                    Show this message and exit.\n```\n',
    'author': 'Irene',
    'author_email': 'irene@wn.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import os


class GitlabConfig:
    env_mapping = {
        "CI_SERVER_URL": "server_url",
        "CI_MERGE_REQUEST_IID": "mr_id",
        "CI_PROJECT_ID": "project_id",
        "CI_JOB_NAME": "job_name",
        "GITLAB_PRIVATE_TOKEN": "private_token"
    }

    converters = {
        "mr_id": int,
        "project_id": int
    }

    def __init__(self, **kwargs):
        self.get_from_environ()

        for attr_name, attr_value in kwargs.items():
            if attr_value is None:
                continue

            if attr_name in self.env_mapping.values():
                setattr(self, attr_name, attr_value)

    def get_from_environ(self):
        for env_name, attr_name in self.env_mapping.items():
            attr_value = os.environ.get(env_name)
            if attr_value and attr_name in self.converters:
                attr_value = self.converters[attr_name](attr_value)
            setattr(self, attr_name, attr_value)



#!/usr/bin/env python
import sys

import click

from gitlab_mr_note.config import GitlabConfig
from gitlab_mr_note.core import post_comment


@click.command()
@click.option("--server-url", "-s", help="Server URL of gitlab instance")
@click.option("--mr-id", "-m", help="ID of the MR to comment on")
@click.option("--project-id", "-p", help="ID of the Project")
@click.option("--job-name", "-j", help="Job Name")
@click.option("--private-token", "-t", help="Private Token")
def main(*args, **kwargs):
    gitlab_config = GitlabConfig(**kwargs)
    post_comment(gitlab_config, sys.stdin.read())


if __name__ == "__main__":
    main()
